#!/bin/sh

# Author Janne Karhunen, Pekka Lundstrom
# Copyright (C) Nokia Oyj 2010

CPULEVEL=50
POLL_INTERVAL=1
MAX_POLLS=10
HIGH_PRIORITY_MARKER="/var/run/high_pri_load_launcher_running"

PREV_TOTAL=0
PREV_IDLE=0
DIFF_USAGE=0
POLLS=0

# $1 is the priority (0=normal, 1=high)
# $2 is number of apps to be started in one launch
# $3 is the path prefix
# $4...n apps to be started

if [ $# -lt 3 ]; then
  echo "load-launcher.sh need argument list to start!"
  exit 1 
fi
HIGH_PRIORITY=$1; shift
NUM_LOADS=$1; shift
PATH_PREFIX=$1; shift

if [ $HIGH_PRIORITY -eq 1 ];then
    touch $HIGH_PRIORITY_MARKER
fi

#echo "load_launcher: (`cut -d " " -f 1 /proc/uptime`) starting pri=$HIGH_PRIORITY, num-load=$NUM_LOADS, path=$PATH_PREFIX"

get_load()
{
  CPU=`cat /proc/stat | grep '^cpu '` # Get the total CPU statistics.
  TOTAL=0
  I=0
  # Calculate the total CPU time and get idle time
  TOTAL=0
  for VALUE in $CPU; do
      if [ $I -ne 0 ]; then
          let "TOTAL=$TOTAL+$VALUE"
          if [ $I -eq 4 ]; then
              IDLE=$VALUE
          fi
      fi
      let I=$I+1
  done
  # Calculate the CPU usage since we last checked.
  let "DIFF_IDLE=$IDLE-$PREV_IDLE"
  let "DIFF_TOTAL=$TOTAL-$PREV_TOTAL"
  let "USAGE_PC=(1000*($DIFF_TOTAL-$DIFF_IDLE)/$DIFF_TOTAL+5)/10"

  # Remember the total and idle CPU times for the next check.
  PREV_TOTAL="$TOTAL"
  PREV_IDLE="$IDLE"
}

while [ ! -z "$1" ]; do
    # If we are low priority launcher, we must wait high one to exit
    while [ -f $HIGH_PRIORITY_MARKER -a $HIGH_PRIORITY -eq 0 ]; do
         #echo "load_launcher: (`cut -d " " -f 1 /proc/uptime`) low-prio yielding"
         sleep $POLL_INTERVAL
         POLLS=0
    done

    if [ $HIGH_PRIORITY -eq 1 ]; then
        # For high priority jobs, we don't care about cpu level
        # We just launch once a second, no matter what
        USAGE_PC=0
    else
        get_load
    fi

    if [ $USAGE_PC -lt $CPULEVEL ]; then
        #echo "load_launcher: (`cut -d " " -f 1 /proc/uptime`) load $USAGE_PC%, starting next $NUM_LOADS job(s)"
        i=0;
        while [ $i -lt $NUM_LOADS ]; do
            if [ ! -z "$1" ]; then
                start $PATH_PREFIX/$1 || true
                shift
            else 
                break
            fi
            let i=$i+1
        done
        POLLS=0
    else 
        let POLLS=$POLLS+1
        if [ $POLLS -gt $MAX_POLLS ]; then
            #echo "load_launcher: (`cut -d " " -f 1 /proc/uptime`) load $USAGE_PC%, forced busy start for next job"
            start $PATH_PREFIX/$1 || true
            POLLS=0
            shift 
        fi
    fi
    sleep $POLL_INTERVAL
done

if [ $HIGH_PRIORITY -eq 1 ];then
    rm -f $HIGH_PRIORITY_MARKER
fi
